import java.util.StringTokenizer;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import reservation.Mode;
import reservation.*;
import reservation.Command;
import reservation.CustomDialog;
import reservation.system.functions.Functions;
import gnu.getopt.LongOpt;
import gnu.getopt.Getopt;

/**
 * <B>The main entry to the Flight Reservation System </B>
 * The class FlightSystemUser implements the entry point of the program. The main () method receives and process the different parameters passed when the program is launched. The user can choose between read the command from the console, from a file, or from a graphic interface. The command is then analysed and the appropriate class is instantiated.
 * @author Frederic Bidon and Mathieu Texier
 * @version 1.0
 * @see FlightSystem
 */
class FlightSystemUser {
    
    /**
    * Display the usage for the program.
    */
    static public void usage (){
        String usage = "\r\n"+
        "Flight System v1.0\r\n" +
        "\r\n"+
        "Usage: java FlighSystemUser [option] {mode}+\r\n" +   
        "\r\n"+
        "Where mandatory mode is a combination of:\r\n"+
        " -g[n]\t--graphic\r\n" +
        "\t graphical mode: \r\n"+
        "\t A Gui instance is launched to interface the program.\r\n"+
        "\t n is an integer (between 1 and 5) coresponding to the number of windows"+
        "\t launched.\r\n\r\n"+
        " -c\t--command\r\n" +
        "\t command line: \r\n"+
        "\tThe command could be written on a terminal and processed by the program \twhen enter is pressed.\r\n"+
        " -f <file>\t--file <file>\r\n" +
        "\t input from a file:\r\n"+
        "\t the <file> containing the command is processed by the system.\r\n" +
        "\r\n" +
        "Option include :\r\n"+
        " -v[n]\t--verbose\r\n" +
        "\t verbose mode: \r\n" +
        "\t n is the degree of verbosity (between 1 and 4) of the program.\r\n" +
        "\t In command line this option will provide more explanations. \r\n" +  
        "\t 1 corespond to the quiet mode\r\n" +
        "\t 2 will display normal output\r\n" +
        "\t 3 will add the state of the Flight System after each request\r\n" +
        "\t 4 will print the stack is printed for each execption thrown.\r\n\r\n" +
        "Example:\r\n" +
        " The following instruction will process the file\r\n" + 
        " Filename.txt in verbose mode and then will leave the hand in\r\n" +
        " the console for more instruction." +
	"\tJava FlightSystemUser -v -f filename.txt --command\r\n" +
	"This instruction will launch the command mode and one exit\r\n" +
	" the graphical mode"+	
	"\tJava FlightSystemUser -c -g\r\n\r\n" +
        "The command syntax:\r\n";
        usage += Functions.usage();
        usage += "\r\n"+
        "Example of request command message:\r\n" +
	"If you want to create a new flight, you may write in a first time \r\n"+
	"the command create, then the name of the flight (for example SK234), \r\n"+
	"the number of rows (for example 35) and the number of seat for each row \r\n"+
	"(for example 6).\r\n"+
	"Then the command line is as follows:\r\n"+ 
	"\t create SK234 35 6.\r\n\r\n"+
	"Note: All the characters of the command are converts in lowercase. \r\n"+
	"Therefore, you can write the name with something else case.\r\n"+  
	"For example, this command is also correct: Create SK234 35 6.\r\n"+
	"The fallowing entries are erroneous command:\r\n"+
	"\t create SK-235 10 2\r\n"+
	"\t create SK235 120 12\r\n"+
	"In a first case, a special character (forbidden for all the arguments) \r\n"+
	"is inserted in the flight's name. \r\n"+
	"In the second case, the syntax is correct but the two numbers are out of range:\r\n"+ 
	"The maximum size for a flight is 100 rows with a rows length of 10 seats.\r\n"+
        "\r\n"+
        "Java Programming : course 02215\r\n" +
        "author: Frederic Bidon, Texier Mathieu\r\n" +
        "DTU\r\n";
        System.out.println(usage);
    }
    
    /**
    * Execute the operation's mode of the system chosen by the user.
    */
    static public void main (String[] argv) {

        LongOpt[] longopts = new LongOpt[5];
        StringBuffer sb = new StringBuffer ();
        
        longopts[0] = new LongOpt ("verbose", LongOpt.OPTIONAL_ARGUMENT, sb, 'v');
        longopts[1] = new LongOpt ("file", LongOpt.REQUIRED_ARGUMENT, sb, 'f');
        longopts[2] = new LongOpt ("command", LongOpt.NO_ARGUMENT, null, 'c');
        longopts[3] = new LongOpt ("graphical", LongOpt.OPTIONAL_ARGUMENT, sb, 'g');
        longopts[4] = new LongOpt ("help", LongOpt.NO_ARGUMENT, null, 'h');
        
        
        Getopt g = new Getopt ("FlightSystemUser", argv, "-v::f:cg::h:", longopts);
        g.setOpterr (false);
        
        BufferedReader in = new BufferedReader (new InputStreamReader (System.in));
        int VerboseMode = Mode.NORMAL;

        Mode mode = null;
        int c = g.getopt ();
        do
            switch (c) {
                case 'v' : // Verbose Mode
                    String level = g.getOptarg ();
                    if (level != null) {
                    if (Functions.checkInteger (level, Mode.QUIET, Mode.DEBUG))
                        {VerboseMode = Integer.parseInt(level);}
                    else {System.err.println("The verbosity Level is comprised between 1 (-quit) and 4 (-debug)");}
                    }
                    else VerboseMode = Mode.NORMAL;
                    System.out.println ("Verbosity level : " + VerboseMode);
                    break;
                case 'c' : // command Line
                    System.out.println ("Command Line");
                    mode = new Command (in,VerboseMode);
                    mode.process ();
                    break;
                    
                case 'f' : // File
                     
                    String fileName = g.getOptarg ();
                    try{
                        in = new BufferedReader (new InputStreamReader (new FileInputStream (fileName)));
                    } catch (FileNotFoundException e){ System.err.println ("The file doesn't exist.");}
                    mode = new Command (in,VerboseMode);
                    mode.process ();
                    break;
                    
                case 'g' : // graphical mode
                    String numberOfInstance = g.getOptarg ();
                    int number = 1;
                    if (numberOfInstance != null) {
                        if (Functions.checkInteger (numberOfInstance, 1, 5))
                            {number = Integer.parseInt(numberOfInstance);}
                        else {System.err.println("The number of instance is limited between 1 and 5");}
                    }
                    System.out.println ("Graphical mode " + number + " instance(s).");
                    
                    for (int i=0;i<number;i++) {
                        mode= new CustomDialog (VerboseMode);
                        mode.process ();          
                    }
                    break;
                    
                case 'h' : // help
                    System.out.println ("help");
                    usage ();
                    break;
                case -1:
                    String Help = "Flight System v1.0\r\n";
                    Help += "\r\n";
        	    Help += "Right usage: java FlighSystemUser [option] {mode}+\r\n";
        	    Help += "Execute the program with argument -h for help.";
                    System.out.println(Help);
                    break;
                default:
                    System.out.println ("Nonvalid argument.");
                    usage ();
                    System.exit(1);
                    break;
            }
            while ((c = g.getopt ()) != -1);
            
            for (int i = g.getOptind (); i < argv.length ; i++) {
                System.out.println ("Nonvalid argument: " + argv[i] + "\n");
                usage ();
                System.exit(1);
            }
    }
}
